function [E] = build_e(geometry, sectfiles, properties, wind_file,units)

%===================================================================
% build_e   4.3   93/07/01   Copyright (C) Phil Morgan 1991.
%
% [E] = build_e(geometry, sectfiles, properties)
%
% DESCRIPTION:
%    Constructs ekman flux matrix E from "ecmwf" wind stress data.
%    Uses binned data files for the mean thermo property to ekman depth De.
%    Mean thermo prop calcualted by an exponential decay from sea surface
%    to a depth of 60db.
%
% INPUT:
%    geometry   = DIM( nboxes, nsections )
%    sectfiles  = DIM( nsections, * )
%    properties =  DIM(nproperties,*)
%    wind_file  = wind climatology data 

% OUTPUT:
%    E          = matrix of ekman property fluxes .

% DATAFILES:
%    Uses ecmwf wind stress data file: see getecmwf.m for details
%
% EXAMPLE:  E = build_e(geometry, sectfiles, properties, wind_file)
%
% CALLER:   dobox
% CALLEE:   charword.m, loadvars.m, getwind.m, ekman_new.m,  weight_expdecay.m,
%           weight_mean.m
%
% AUTHOR:   Phil Morgan 91-11-29
%
% FUTURE:   Allow the effective depth of ekman to be varied.
%
% Modified: Bernadette Sloyan 95-4-27
% Modified: Bernadette Sloyan 00-7-13 Use ekman_new to 
%           calculate Ekman flux. This input an addition
%           variable "units"
%
%==================================================================
% @(#)build_e.m  Revision: 4.3  Date: 93/07/01
%--------------------------------------------------------------------


EkmanBinDepth = 7;   % bin depth of limit of ekman influence
weights       = expdecay( EkmanBinDepth );


% SET FOR STEP THRU EACH SECTION
[nboxes, nsections] = size(geometry);
[nproperties, n]    = size(properties);

E = [];

for isect = 1:nsections
  sectfile = sectfiles(isect,:);
  sectfile = sectfile( find( sectfile~=blanks(1) ) );  % strip blanks off name

  disp(['build_e: section ' num2str(isect) ' from file {' sectfile '}'])

  [lat,lon]   = loadvars(sectfile,'lat','lon');
  [Area_mass] = loadvars(sectfile,'Area_mass');

  % GET EKMAN FLUX "q" FOR SECTION
  [taux, tauy]  = getwind(lat, lon, wind_file);
  [sect_q]      = ekman_new(taux, tauy, lat, lon, units);
  sect_q        = change(sect_q,'==',NaN,0);  % because some wind missing
  
  % STEP THRU EACH COLUMN IN SECTION AND CALC WEIGHTED MEAN THERMO PROPERTY
  [nlayers, npairs ] = size(Area_mass);
  [mrows,ncols]      = size(Area_mass);

  % FIND ROW (LAYER NUMBER) WHERE FIRST VALID DATA
  [values,ekman_rows]=firstgood(Area_mass,0);

  %------------------------------------------------
  % CALC EKMAN PROPERTY TRANSPORT FOR EACH PROPERTY
  %------------------------------------------------  
  E_box = [];
  disp('Calculating Ekman property flux for properties...')
  for iprop = 1:nproperties
      PropName = [charword( properties(iprop,:) )];
      disp(['      ' PropName])
        
      if iprop == 1
         % mass property
	 mean_prop = ones(1,npairs);
      else
         % binPair_prop=loadvars(sectfile,'binPair_sal');  	
	 Var          = ['binPair_' PropName];
         Expression   = ['loadvars(sectfile,Var)'];
         command      = ['binPair_prop = ' Expression ';'];
         eval(command)

         mean_prop = weight_mean(binPair_prop(1:EkmanBinDepth ,:) , weights );
      end %if
      
      % FILL IN A "SECTION" ARRAY OF EKMAN FLOW & MEAN PROPS ONLY AT 
      % FIRST VALID PT
      E_sect_prop  = zeros(size(Area_mass));
      for icol = 1:ncols
         if ~isnan( ekman_rows(icol) ) 
            E_sect_prop( ekman_rows(icol),icol) = mean_prop(icol)*sect_q(icol);
         end %if
      end %for
     
      % debug: E_sect_propx = E_sect_prop
      % CALC BOX EQNS FOR SECTION
      E_box  = [E_box; E_sect_prop; sum(E_sect_prop)];

  end %for "iprop"
  
  % FILL IN A FOR ALL BOXES IN THIS SECTION
  E_sect = [];
  for ibox = 1:nboxes
    E_sect = [E_sect; geometry(ibox,isect)*E_box];
  end
  
  % FILL A FOR EACH SECTION 
  E = [E E_sect];

end %for "isect"
return
%--------------------------------------------------------------------

