% defindices.m
%
% Identify the components of A, x and b in 
% DOBOX format, using the data structure 
% pointers (see DOBOX manual, pg. 24).
%
% x(indv(i))   	reference velocity at stn. pair i
% x(indw(i,j,k))  equivalent diapycnal velocity
%		at level i of property j, box k.
% b(indb(i,j,k))  layer i flux of property j, box k.
%		(i=nlayers+1 for vertical integral)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create indices for unknowns %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% (column) indices for ref. velocity
indv=(1:nAbase)';
% (column) indices for w. 
for k=1:nboxes;
  for j=1:nproperties;
    jj=(k-1)*nproperties+j;
    indw(:,j,k)=(  nAbase+1+(jj-1)*(nlayers-1) : ...
        nAbase+jj*(nlayers-1) )';
  end;
end;

% indices for air-sea forcing corrections
% (tacked on after w* terms): corrections
% are for heat and freshwater flux only;
% corresponding density flux corrections are
% directly calculated from F*,h and F*,fw
%  x(indF(:,1,:)) : correction to H flux
%  x(indF(:,2,:)) : correction to FW flux
% NOTE: these terms are defined on the
% glevels grid, not on glayers.
indF=NaN*ones(nlayers-1,2,nboxes);
ct=1;
for k=1:nboxes;
  for j=1:2;
    for i=1:nlayers-1;
      indF(i,j,k)=max(max(max(indw)))+ct;
      ct=ct+1;
    end;
  end;
end;

% indices for Ekman transport corrections
% (one multiplicative factor for the net
% Ekman transport across each section, 
% applied to all layers and all properties)
indE=NaN*ones(nsections,1);
ct=1;
for k=1:nsections;
  indE(k)=max(max(max(indF)))+ct;
  ct=ct+1;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% done creating indices for unknowns.  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create indices for property fluxes %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% (row) indices for property fluxes;
% indb(1:nlayers,j,k) are the layer fluxes
% of property j in box k; 
% indb(nlayers+1,j,k) is the vertically-
% integrated flux of property j in box k.
indb=NaN*ones(nlayers+1,nproperties,nboxes);
for k=1:nboxes;
  for j=1:nproperties;
    jj=(k-1)*nproperties+j;
    indb(:,j,k)=( 1+(jj-1)*(nlayers+1): ...
        jj*(nlayers+1) )';
  end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% done creating property flux indices %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nA>nAbase)	% if diapycnal velocities are included
  % Extract mean level areas and properties from A.
  % meanprop(i,j,k) is the mean value of property j
  % at level i, in box k.  For j=1, this is the level's area.
  % (Note: this could alternatively be done by loading
  % the values from the mprop_atl*.mat files.)

  meanprop=NaN*ones(nlayers-1,nproperties,nboxes);
  for k=1:nboxes;
    for j=1:nproperties;
      meanprop(:,j,k)=diag(A(indb(1:nlayers,j,k),indw(:,j,k)));
    end;
  end;

  % define level_area, the area of the layers
  level_area=meanprop(:,1,:);
  level_area=reshape(level_area,[nlayers-1,nboxes]);
  q=find(level_area==0);
  level_area(q)=NaN;

  % divide by level area to get mean property 
  % values (ones for volume, salinity anomaly in
  % psu, temperature in degC., etc.)
  for k=1:nboxes
    for j=1:nproperties
      meanprop(:,j,k)=meanprop(:,j,k)./level_area(:,k);
    end;
  end;

  q=find(meanprop==0);
  meanprop(q)=NaN; 

end;

