% gauss.m
%
% Use Gauss-Markov estimation to solve the
% system Ax + n = y.
%
% Run after correcting y for air-sea forcing
% and using DOBOX to add diapycnal flux terms
% to A, x.
%
% fixed mixing: specific to 48N variability
% study.  Specify values of x(indw) for subpolar box.
% To find the code specific to this modification,
% do a search on "fixed mixing"

figure(1);
checkgeo(file_geom);
pause(.1);

masslayererror=2;	% layer mass error (Sv)
ms_mf=2;      		% minsalamerror (psu); 35 in orig.
mindiaflux=2;		% minimum dia.vol.flux (Sv)

includemixing_ap=0;	% set to 1 to include drD_ap

consSILCAT=1;		% set to 1 to conserve net silicate
SILCATerror=100;		% silicate conserved to this level (kmol/s)

masslayererror=masslayererror*ones(size(glayers));
% magnify the layer errors in layers 1:6 according
% to seasonal fluctuations in Florida Strait sections
masslayererror(1:6)=masslayererror(1:6).* ...
	[6 5.5 5 4.5 4 2]';
% (this is strictly for masslayererror=1/2 Sv)

masslayererror=masslayererror*1e6;
mindiaflux=mindiaflux*1e6;

% fluxunits: units of terms in b, A*x (after dividing
%	by fluxdivfact).  Untouched units are
%       SI: volume is m^3/s, salam is 1e-3 psu m^3/s,
%	potmp is dgC m^3/s.  Use a mean value of 
%	1e15/(rho*C_p) for potemp to get PW.
fluxunits=['   Sv  '; ...
	   ' psu Sv'; ...
	   '   PW  ']; 
fluxdivfact=[1e6; 1e-3*1e6; 1e15/3985/1026];
propnames=['mass';'salt';'temp'];
propunits=['10^{6} km^2'; ...
	'    psu    '; ...
	'  ^\circC  '];
propdivfact=[1e12;1e-3;1];
propcolr=[[0 0 1];[0 .5 0];[1 0 0]];

% load positions of station pairs
lon2=[];lat2=[];theta2=[];sectlabel=[];R2=[];vmag=[];
for i=1:nsections;
  eval(['load ',sectfiles(i,:),' lon lat']);
  sectt=['refvel_', ...
	sectfiles(i,length(directory)+1: ...
	size(sectfiles,2))];
  eval(['load ',sectt,' Vmag']);

  q=find(abs(diff(lon))>90);	% +-180
  l2=(lon(1:length(lon)-1)+diff(lon)/2)';
  lon(find(lon<0))=lon(find(lon<0))+360;
  l2(q)=(lon(q-1)+lon(q))/2;
  if (l2(q)>180) l2(q)=l2(q)-360; end;
  lon2=[lon2;l2]; clear l2;
  lat2=[lat2;(lat(1:length(lat)-1)+diff(lat)/2)'];
  [R,theta]=sw_dist(lat,lon,'km');
  R2=[R2;R'];
  theta2=[theta2;theta'];
  sectlabel=[sectlabel;i*ones(length(lon)-1,1)];
  vmag=[vmag;Vmag'];
end;	

lon=lon2;clear lon2;
lat=lat2;clear lat2;
R=R2*1000;clear R2;	% R [m]
theta=theta2*2*pi/360;clear theta2;	
			% theta [rad]
clear Vmag sectt;

%keyboard;

% define glayers, a vector containing the mean
% gamma of the model layers.
load glevels;
glayers=[glevels(1)-.5*(glevels(2)-glevels(1)); ...
	glevels(1:length(glevels)-1)+diff(glevels)/2; ...
	last(glevels)+last(diff(glevels))/2];
layers=(1:nlayers)';

% define the indices for the variables.
defindices;

disp('Conserved properties are hard-coded. ');
disp('  CHECK: property 1 should be mass,');
disp(['                       and is ',properties(1,:)]);
disp('  CHECK: property 2 should be salam,');
disp(['                       and is ',properties(2,:)]);
disp('  CHECK: property 3 should be potmp or temp,');
disp(['                       and is ',properties(3,:)]);
meanprop_nonans=meanprop;
level_area_nonans=level_area;
for boxnum=1:nboxes;
  for propnum=1:nproperties;
    meanprop_nonans(:,propnum,boxnum)=fillnans( ...
	meanprop(:,propnum,boxnum),0);
  end;
end;
q=find(isnan(level_area));
level_area_nonans(q)=0;

% include adjustments to density conservation
% (i.e. w*_rho terms) in budgets of other
% properties - this represents advection
% across a mean property gradient driven by
% net buoyancy fluxes, in addition
% to the diffusive (eddy) fluxes parameterized
% by the w* terms.
for boxnum=1:nboxes;
  for propnum=2:nproperties;
    for laynum=1:nlayers;
      A(indb(laynum,propnum,boxnum), ...
	indw(:,1,boxnum))= ...
	A(indb(laynum,propnum,boxnum), ...
	indw(:,propnum,boxnum));
    end;
  end;
end;

% Define matrices which contain the surface fluxes and values
%
%  AirSea(layer,property,box) : air-sea input
%  eAirSea(layer,property,box) : error bar range of AirSea
%  EkmanI(layer,property,box) : internal (within each box) 
%				Ekman flux
%  EkmanE(layer,property,box,section)
%			      : external (across-section)
%				Ekman flux
%  SSValue(layer,property,box) : sea surface area, temp, salinity
%  Si_Input(box) : river input of silica (kmol/s)

Fdens=NaN*ones(nlayers+1,nboxes);
AirSea=NaN*ones(nlayers,nproperties,nboxes);
eAirSea=AirSea;EkmanI=AirSea; SSValue=AirSea;
EkmanE=NaN*ones(nlayers,nproperties,nboxes,nsections);
EkmanAdjust=zeros(nsections,1);
Si_Input=NaN*ones(nboxes,1);
for boxnum=1:nboxes;
  eval(['load airsea',num2str(boxnum),' M* F* Ek*' ...
	' gamma2 SS* si_flux']);

  Fdens(:,boxnum)=Fm(1,:)';

  AirSea(:,1,boxnum)=Ffw(1,:);
  AirSea(:,2,boxnum)=zeros(size(Ffw(1,:)));
  AirSea(:,3,boxnum)=Fh(1,:);

  eAirSea(:,1,boxnum)=Ffw(2,:);
  eAirSea(:,2,boxnum)=Mm(2,:);
  eAirSea(:,3,boxnum)=Fh(2,:);

  EkmanI(:,1,boxnum)=Mmek;
  EkmanI(:,2,boxnum)=Msek;
  EkmanI(:,3,boxnum)=Mhek;

  EkmanE(:,1,boxnum,:)=reshape(EkmanInM, ...
	[nlayers,1,1,nsections]);
  EkmanE(:,2,boxnum,:)=reshape(EkmanInS, ...
	[nlayers,1,1,nsections]);
  EkmanE(:,3,boxnum,:)=reshape(EkmanInH, ...
	[nlayers,1,1,nsections]);

  EkmanAdjust=max([EkmanAdjust, ...
	EkmanInM_range./abs(nansum(EkmanInM)'+1) ...
	]')';

  SSValue(:,1,boxnum)=SSArea;
  SSValue(:,2,boxnum)=SSSbar;
  SSValue(:,3,boxnum)=SSTbar;

  Si_Input(boxnum)=si_flux;

end;
clear Mm* Ffw* Fh* Ms* Mh* EkmanInM EkmanInS EkmanInH;
clear SSTbar SSSbar SSArea;

% Find interior sections (defined as sum(geometry,1)==0)
% and insure net Ekman flux out of one box equals flux
% into the other.   This won't be true by default due to
% extrapolation of winds and changes in surface gamma
% across the section.  THIS ROUTINE ASSUMES A SECTION
% CAN BE SHARED BY TWO AND ONLY TWO BOXES.
sectint=find(sum(geometry,1)==0);
for i=1:length(sectint);
  EkM=[];EkS=[];EkH=[];
  for boxnum=1:nboxes;		% calculate mean Ekman flux
    EkM=[EkM,EkmanE(:,1,boxnum,sectint(i)).* ...
	geometry(boxnum,sectint(i))];
    EkS=[EkS,EkmanE(:,2,boxnum,sectint(i)).* ...
        geometry(boxnum,sectint(i))];
    EkH=[EkH,EkmanE(:,3,boxnum,sectint(i)).* ...
        geometry(boxnum,sectint(i))];
  end;
  EkM=sum(EkM,2)/2;
  EkS=sum(EkS,2)/2;
  EkH=sum(EkH,2)/2;
  for boxnum=1:nboxes;		% assign mean to each section
    if (geometry(boxnum,sectint(i)))
      EkmanE(:,1,boxnum,sectint(i))= ...
	EkM/geometry(boxnum,sectint(i));
      EkmanE(:,2,boxnum,sectint(i))= ...
        EkS/geometry(boxnum,sectint(i));
      EkmanE(:,3,boxnum,sectint(i))= ...
        EkH/geometry(boxnum,sectint(i));
    end;
  end;
end;
clear sectint EkM EkS EkH;

% identify which layers exist in each box
% using the bounding hydrography around each box
% and the interior air-sea forcing.
defined_layers=zeros(length(glayers),nboxes);
for boxnumber=1:nboxes;
  q=find(abs(geometry(boxnumber,:)));
  for i=1:length(q);
    eval(['load ',sectfiles(q(i),:),' thick']);
    thick=sum(thick')';
    q2=find(thick>0 | (AirSea(:,3,boxnumber)~=0));
    defined_layers(q2,boxnumber)=1;
  end; % loop over sections bounding box
end; % loop over boxnumber
clear thick q q2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define the a priori diapycnal advection %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~includemixing_ap
  drD_ap=zeros(nlayers+1,nproperties,nboxes);
  edrD_ap=drD_ap;
end;
drD_apDivA=drD_ap;
edrD_apDivA=edrD_ap;
% convert drD_apDivA to units of m/s (same as w* terms)
for boxnum=1:nboxes;
  for propnum=1:nproperties;
    divfact=meanprop_nonans(:,propnum,boxnum).* ...
	level_area(:,boxnum);
    drD_apDivA(2:nlayers,propnum,boxnum)= ...
        drD_apDivA(2:nlayers,propnum,boxnum)./divfact;
    edrD_apDivA(2:nlayers,propnum,boxnum)= ...
     	edrD_apDivA(2:nlayers,propnum,boxnum)./abs(divfact);
  end;
end;
q=find(isnan(drD_apDivA));
drD_apDivA(q)=0;
edrD_apDivA(q)=0;
% define W_ap: given by prior Fdens and mixing
W_ap=zeros(nlayers+1,nboxes);
for boxnum=1:nboxes;
  ind=2:nlayers;
  W_ap(ind,boxnum)=-Fdens(ind,boxnum)./ ...
	level_area(:,boxnum) + ...
	drD_apDivA(ind,1,boxnum);
end;
q=find(isnan(W_ap));
W_ap(q)=0;
clear drD_apDivA;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now include the air-sea formation,    %
% Ekman fluxes and a priori mixing      %
% in vector b				%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

includeEkmanI=0;
disp('Including air-sea fluxes in vector b.');
for propnum=1:nproperties;
  for boxnum=1:nboxes;

    % include Ekman fluxes
    b(indb(1:nlayers,propnum,boxnum))= ...
        b(indb(1:nlayers,propnum,boxnum)) - ( ...
	includeEkmanI*EkmanI(:,propnum,boxnum)+ ...
        sum(EkmanE(:,propnum,boxnum,:),4)   );
    b(indb(nlayers+1,propnum,boxnum))= ...
        b(indb(nlayers+1,propnum,boxnum)) - nansum( ...
        includeEkmanI*EkmanI(:,propnum,boxnum)+ ...
        sum(EkmanE(:,propnum,boxnum,:),4)   );

    % include property convergence/divergence
    % driven by a priori diapycnal advection
    b(indb(1:nlayers,propnum,boxnum))= ...
	b(indb(1:nlayers,propnum,boxnum)) ...
	-diff(        W_ap(:,boxnum).* ...
           [0;level_area_nonans(:,boxnum);0].* ...
           [0;meanprop_nonans(:,propnum,boxnum);0] );

    % include direct air-sea inputs 
    % (AirSea for salt is zero).
    b(indb(1:nlayers,propnum,boxnum))= ...
	b(indb(1:nlayers,propnum,boxnum)) - ...
	AirSea(:,propnum,boxnum);
    b(indb(nlayers+1,propnum,boxnum))= ...
	b(indb(nlayers+1,propnum,boxnum)) - ...
	nansum(AirSea(:,propnum,boxnum));

  end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Include adjustments to air-sea forcing %
% in the set of conservation equations   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Including adjustments to air-sea fluxes in A and x.');

A=[A,zeros(mA,(nlayers-1)*2*nboxes)];

Fcorr_multfact=NaN*ones(nlayers-1,2,nboxes);
dg=diff(glayers);

for k=1:nboxes;

  % add heat flux corrections to heat budget
  % (remember, corrections are defined on glevels
  %  grid; add 1/2 from level i-1, 1/2 from level i.)
  %  for top/bottom layers, use 1/2 value at closest level.)
  A(indb(1,3,k),indF(1,1,k))=.5;
  A(indb(2:nlayers-1,3,k),indF(1:nlayers-2,1,k))= ...
	.5*diag(ones(nlayers-2,1));
  A(indb(2:nlayers-1,3,k),indF(2:nlayers-1,1,k))= ...
        .5*diag(ones(nlayers-2,1));
  A(indb(nlayers,3,k),indF(nlayers-1,1,k))=.5;
  % add sum of layer corrections to total heat budget
  A(indb(nlayers+1,3,k),indF(:,1,k))=ones(1,nlayers-1);

  % add freshwater flux corrections to volume budget
  A(indb(1,1,k),indF(1,2,k))=.5;
  A(indb(2:nlayers-1,1,k),indF(1:nlayers-2,2,k))= ...
        .5*diag(ones(nlayers-2,1));
  A(indb(2:nlayers-1,1,k),indF(2:nlayers-1,2,k))= ...
        .5*diag(ones(nlayers-2,1));
  A(indb(nlayers,1,k),indF(nlayers-1,2,k))=.5;
  % add sum of layer corrections to total volume budget
  A(indb(nlayers+1,1,k),indF(:,2,k))=ones(1,nlayers-1);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % construct a density flux correction from the
  % heat and freshwater flux corrections, and include
  % this advection in the property budgets.
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  Tbar=interp1(glayers,fillnans(SSValue(:,3,k),0), ...
	glevels,'linear');
  Sbar=interp1(glayers,fillnans(SSValue(:,2,k),0), ...
	glevels,'linear');
  rho=sw_dens(Sbar,Tbar,0);
  rho0=sw_dens(zeros(size(Tbar)),Tbar,0);
  alpha=sw_alpha(Sbar,Tbar,0);
  beta=sw_beta(Sbar,Tbar,0);

  % density flux from heat flux
  Fcorr_multfact(:,1,k)=-alpha.*rho./dg;

  % density flux from freshwater flux:
  % multiply by -1 (fw flux has opposite
  % sign of E-P flux) and don't forget
  % factor of SSS/(1-SSS/1000)
  Fcorr_multfact(:,2,k)=-beta.*rho./dg .*Sbar./(1-Sbar/1000);

  % Loop through layers.  For each layer, add the
  % density flux into the layer at level i-1 and
  % remove the flux out of the layer at level i.
  for i=1:nlayers-1;	% remove flux at glevel(i)
    for propnum=1:nproperties;
      A(indb(i,propnum,k),indF(i,1,k))= ...
        A(indb(i,propnum,k),indF(i,1,k)) ...
	-Fcorr_multfact(i,1,k).* ...
	meanprop_nonans(i,propnum,k);
      A(indb(i,propnum,k),indF(i,2,k))= ...
	A(indb(i,propnum,k),indF(i,2,k)) ...
	-Fcorr_multfact(i,2,k).* ...
	meanprop_nonans(i,propnum,k);
    end;
  end;
  for i=2:nlayers;	% add flux at glevel(i-1)
    for propnum=1:nproperties;
      A(indb(i,propnum,k),indF(i-1,1,k))= ...
	A(indb(i,propnum,k),indF(i-1,1,k)) ...
        +Fcorr_multfact(i-1,1,k).* ...
	meanprop_nonans(i-1,propnum,k);
      A(indb(i,propnum,k),indF(i-1,2,k))= ...
	A(indb(i,propnum,k),indF(i-1,2,k)) ...
        +Fcorr_multfact(i-1,2,k).* ...
	meanprop_nonans(i-1,propnum,k);
    end;
  end;
  % by construction, these terms sum to zero for
  % vertically-integrated property balances.

  % done adding density flux corrections
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end;
nA=size(A,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Include corrections to air-sea forcing %
% in the set of conservation equations   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Including corrections to Ekman transports in A and x.');
A=[A,zeros(mA,nsections)];
for sectnum=1:nsections;
  for i=1:nlayers+1;
    for j=1:nproperties;
      for k=1:nboxes;
	if (i<nlayers+1)
          A(indb(i,j,k),indE(sectnum))= ...
	    EkmanE(i,j,k,sectnum);
        else
          A(indb(i,j,k),indE(sectnum))= ...
            nansum(EkmanE(:,j,k,sectnum));
        end;
      end;
    end;
  end;
end;
nA=size(A,2);
disp(' ');

%%%%%%%%%%%%%%%%%%%%%%%
% Define model errors %
%%%%%%%%%%%%%%%%%%%%%%%

% allocate memory for modelerror
modelerror=NaN*ones(nlayers+1,nproperties,nboxes);

for (boxnum=1:nboxes);

  %%%% mass errors %%%%%
  % set layer errors to masslayererror
  modelerror(1:nlayers,1,boxnum)=masslayererror;

  % assign a very small error to non-existing layers
  modelerror(find(~defined_layers(:,boxnum)), ...
        1,boxnum)=.01e6;

  % assign masstotalerror 
  modelerror(nlayers+1,1,boxnum)=masstotalerror(boxnum);

  % identify the sections surrounding this box
  sects=find(geometry(boxnum,:));
  % loop through sections, loading layer-mean
  % properties
  std_potmp=[];std_salam=[];
  for sectnum=1:length(sects);
    eval(['load ',sectfiles(sects(sectnum),:), ...
	' LayerPair_potmp LayerPair_salam;']);
    std_potmp=[std_potmp,LayerPair_potmp];
    std_salam=[std_salam,LayerPair_salam];
  end;
  % remove 0s (empty pairs in LayerPair_*)
  % and calculate layer std.deviations.
  q=find(~std_potmp);std_potmp(q)=NaN;
  q=find(~std_salam);std_salam(q)=NaN;
  mean_salam=abs(nanmean(std_salam')');
  mean_potmp=abs(nanmean(std_potmp')');
  std_potmp=fillnans(nanstd(std_potmp')',0);
  std_salam=fillnans(nanstd(std_salam')',0);

  mean_salam2=abs(fillnans(interp1(glevels, ...
	meanprop(:,2,boxnum),glayers,'linear'),0));
  mean_potmp2=abs(fillnans(interp1(glevels, ...
        meanprop(:,3,boxnum),glayers,'linear'),0));
  mean_salam=max([mean_salam,mean_salam2]')';
  mean_potmp=max([mean_potmp,mean_potmp2]')';

  % assign (abs(mean) + 2 std)*mass flux error to modelerror
  modelerror(1:nlayers,2,boxnum)= ...
	(abs(mean_salam)+2*std_salam).* ...
	modelerror(1:nlayers,1,boxnum);
  % vert. integrated error: max layer error times
  % ratio of total mass error to layer mass error
  %q=find(modelerror(1:nlayers,2,boxnum)== ...
        %max(modelerror(1:nlayers,2,boxnum)));
  %modelerror(nlayers+1,2,boxnum)= ...
        %modelerror(q(1),2,boxnum) ...
        %* modelerror(nlayers+1,1,boxnum) ...
        %/ modelerror(q(1),1,boxnum);
  % new method: sum the layer errors as if they
  % were independent, and scale by
  % ratio of total mass error to layer mass errors
  modelerror(nlayers+1,2,boxnum)= ...
        sqrt(sum(modelerror(1:nlayers,2,boxnum).^2))  ...
        *masstotalerror(boxnum)/sqrt(sum( ...
	modelerror(1:nlayers,1,boxnum).^2));
  % set floor to error
  q=find(modelerror(1:nlayers,2,boxnum)< ...
	ms_mf/1e3*modelerror(1:nlayers,1,boxnum));
  modelerror(q,2,boxnum)= ...
	ms_mf/1e3*modelerror(q,1,boxnum);
  % if total salam error is < ms_mf/1e3*masstotalerror,
  % set it to this value
  %if (modelerror(nlayers+1,2,boxnum)< ...
	%ms_mf/1e3*masstotalerror(boxnum))
    %modelerror(nlayers+1,2,boxnum)= ...
	%ms_mf/1e3*masstotalerror(boxnum);
  %end;

  %%%% potential temperature errors %%%%
  % error in PW: meanprop(:,3,boxnum)/(1e15/3990/1026).
  % assign (abs(mean) + 2 std)*mass flux error to modelerror
  modelerror(1:nlayers,3,boxnum)= ...
        (abs(mean_potmp)+2*std_potmp).* ...
        modelerror(1:nlayers,1,boxnum);

  % vert. integrated error: max layer error times
  % ratio of total mass error to layer mass error
  %q=find(modelerror(1:nlayers,3,boxnum)== ...
        %max(modelerror(1:nlayers,3,boxnum)));
  %modelerror(nlayers+1,3,boxnum)= ...
	%modelerror(q(1),3,boxnum) ...
	%* modelerror(nlayers+1,1,boxnum) ...
	%/ modelerror(q(1),1,boxnum);
  % new method: sum the layer errors as if they
  % were independent, and scale by
  % ratio of total mass error to layer mass errors
  modelerror(nlayers+1,3,boxnum)= ...
	sqrt(sum(modelerror(1:nlayers,3,boxnum).^2))  ...
	*masstotalerror(boxnum)/sqrt(sum( ...
	modelerror(1:nlayers,1,boxnum).^2));
  % newest method: total heat error is dominated by
  % (equal to) the largest layer error.  This is
  % a weaker constraint than the "new method" above,
  % but is overly optimistic if the upper layer errors
  % are strongly correlated.
  %modelerror(nlayers+1,3,boxnum)= ...
	%max(modelerror(1:nlayers,3,boxnum));

  eval(['clear *_atl',num2str(boxnum)]);
  clear mean_salam* std_salam mean_potmp* std_potmp;

end;

% allocate memory for the noise and model
% variance matrices
Rnn=eye(mA,mA);
Rxx=eye(nA,nA);

% assign squared values to error matrix Rnn
for i=1:nlayers+1;
  for j=1:nproperties;
    for k=1:nboxes;
      Rnn(indb(i,j,k),indb(i,j,k))=modelerror(i,j,k)^2;
    end;
  end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define a priori values for unknowns %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% a priori reference velocities
Rxx(indv,indv)=diag(vmag.^2);

if (0)
  % find where station separation distance
  % is less than corrdist (km); scale the cross-
  % correlation elements of Rxx for these
  % pairs by the a priori vmag values,
  % scaled by ( 1-dist/corrdist).
  corrdist=20;
  for stnpair=1:length(lon);
    dist=NaN*ones(size(lon));
    for j=1:length(lon);
      dist(j)=sw_dist([lat(stnpair) lat(j)], ...
	[lon(stnpair) lon(j)],'km');
    end;
    q=find(dist>0 & dist<corrdist);
    for j=1:length(q);
      Rxx(indv(q(j)),indv(stnpair))= ...
	vmag(q(j))*vmag(stnpair) * (1-dist(q(j))/corrdist);
      Rxx(indv(stnpair),indv(q(j)))= ...
	vmag(q(j))*vmag(stnpair) * (1-dist(q(j))/corrdist);
    end;
  end;
end;

% a priori diapycnic velocities

for boxnum=1:nboxes;

  wmag=NaN*ones(nlayers-1,nproperties);
  for propnum=1:nproperties;

    pfluxrel=-sum(D(indb(1:nlayers,propnum,boxnum),indv),2);
    pfluxek=sum(EkmanE(1:nlayers,propnum,boxnum,:),4);
    Psi=[flipud(cumsum(flipud(pfluxrel+pfluxek)));0];
    % include direct air-sea input
    Psi=Psi+[flipud(cumsum(flipud(AirSea(:,propnum,boxnum))));0];
    % include a priori diapycnal advection
    % (note: "Psi" becomes Psi+drD-F at this step)
    wap=[W_ap(:,boxnum), W_ap(:,boxnum)];
    if (propnum>1);  % include adjustments to advection
      wap=wap+[wadj,-wadj];
    end;
    %note that -W_ap(:,1).*[0;level_area_nonans(:,1);0]=Fdens(:,1);
    Psi=[Psi,Psi]-[[0,0];wap(2:nlayers,:).* ...
	(level_area_nonans(:,boxnum)*ones(1,2) ).* ...
	(meanprop_nonans(:,propnum,boxnum)*ones(1,2));[0,0]];
    clear wap;
    % adjusted Psi will conserve net property, i.e. go to zero
    % at gamma2(1).  Assume these adjustments are evenly-
    % distributed across the layers
    % NOTE: if this is not done for mass, then accumulated 
    %  errors in the relative+ekman can produce large positive
    %  values of Psi in the lightest layers - this is reduced
    %  by the large negative (to lighter layers) F_as, producing
    %  small values of wmag when large ones are needed to
    %  counter F_as (Niiler-Stevenson balance)
    if (1)
      mPsi=nanmean(Psi')';
      q=min(find(mPsi==0 & gamma2>27));
      q2=max(find(~abs(diff(mPsi)) & glayers<27))+1;
      if ~length(q2) q2=1; end;
      mPsi=fillnans((interp1([q2 q],[Psi(q2) Psi(q)], ...
        1:length(gamma2)))',0) *ones(1,2);
      Psi=Psi-mPsi;
      clear mPsi;
    end;

    if (propnum==1) 
      wadj=Psi(:,1)./[NaN;level_area(:,boxnum);NaN];
      q=find(isnan(wadj));wadj(q)=0;
    end;
  
    diafluxmag=abs(-Psi);
    diafluxmag=nanmax(diafluxmag')';

    % drop endpoints to get glevels grid
    diafluxmag=diafluxmag(2:length(diafluxmag)-1);
    wmag(:,propnum)=diafluxmag./level_area(:,boxnum) ...
	./abs(meanprop(:,propnum,boxnum));
  end;  % loop over propnum

  % Allow for a little "slop" in the system: don't force
  % each and every level to have the minimum mixing according
  % to the a priori choices, so that the system has more
  % freedom to adjust from those a priori values and find
  % a better global minimum for mixing.  This "slop" is
  % mindiaflux (minimum diapycnal volume flux) for mass,
  % mindiaflux*(magprop) for other properties.
  sarea=level_area(:,boxnum) * ones(1,nproperties) ...
	.*abs(meanprop(:,:,boxnum));
  magprop=nanmean(wmag.*sarea);
  magprop=magprop./magprop(1);
  wmag=wmag+(mindiaflux*ones(nlayers-1,1)*magprop)./sarea;
  wmag(find(isnan(wmag)))=1e-99;
  % include error bar on a priori mixing
  wmag=wmag+edrD_apDivA(2:nlayers,:,boxnum);
  % allow mixing of heat, salt to be large enough to
  % counter diapcynal advection (e.g. Niiler-Stevenson balance)
 % for propnum=2:nproperties;
    %q=find(wmag(:,propnum)<wmag(:,1));
    %wmag(q,propnum)=wmag(q,1);
  %end;

  % assign wmag^2 to Rxx
  for propnum=1:nproperties;
    Rxx(indw(:,propnum,boxnum),indw(:,propnum,boxnum))= ...
	  diag( wmag(:,propnum).^2 );
  end;

  % air-sea correction terms
  % heat
  %tvals=interp1(glayers,eAirSea(:,3,boxnum), ...
        %glevels,'linear').^2;
  tvals=NaN*ones(size(glevels));
  for ti=1:length(glevels);
    tvals(ti)=max([eAirSea(ti,3,boxnum) ...
	 eAirSea(ti+1,3,boxnum)].^2);
  end;
  q=find(tvals>0);q2=find(tvals==0);
  tvals(q2)=.01*min(tvals(q));
  Rxx(indF(:,1,boxnum),indF(:,1,boxnum))=diag( tvals );

  % freshwater
  %tvals=interp1(glayers,eAirSea(:,1,boxnum), ...
	%glevels,'linear').^2;
  tvals=NaN*ones(size(glevels));
  for ti=1:length(glevels);
    tvals(ti)=max([eAirSea(ti,1,boxnum) ...
         eAirSea(ti+1,1,boxnum)].^2);
  end;
  q=find(tvals>0);q2=find(tvals==0);
  tvals(q2)=.01*min(tvals(q));
  Rxx(indF(:,2,boxnum),indF(:,2,boxnum))=diag( tvals );

end;	% looping over boxnum

% a priori adjustments to net Ekman transport
if (length(EkmanAdjust)==1)
  Rxx(indE,indE)=diag( EkmanAdjust * ones(size(indE))).^2;
else
  Rxx(indE,indE)=diag( EkmanAdjust ).^2;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Add constraints  		%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%% fixed mixing %%%%%%%%%%%%%%%
% constrain mixing in subpolar box to be ensemble mean
% value of wbox2(levels,props,hydr.repeat index).
% Error is 10%
if (1)
boxnum=1;
load ../mixings
wbox2=mean(wbox2,3);
q=find(wbox2==0);wbox2(q)=1e-9;
for i=1:nlayers-1;
  for j=1:nproperties;
    A=[A;zeros(1,size(A,2))];
    A(size(A,1),indw(i,j,boxnum))=1;
    b(length(b)+1)=wbox2(i,j);
    Rnn(length(Rnn)+1,length(Rnn)+1)= ...
	(0.1*wbox2(i,j))^2;
    if (Rxx(indw(i,j,boxnum),indw(i,j,boxnum))< ...
        1.1*abs(wbox2(i,j))^2 )
      Rxx(indw(i,j,boxnum),indw(i,j,boxnum))= ...
        1.1*abs(wbox2(i,j))^2;
    end;
  end;
end;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% add constraints on total (ALL boxes) 
% property budgets
if (nboxes>1 & 0)	% (already done if only one box)
  for propnum=1:nproperties;
    % assign the A-values for box 1
    AT=A(indb(nlayers+1,propnum,1),:);
    % add the A-values for the other boxes
    for boxnum=2:nboxes;
      AT=AT+A(indb(nlayers+1,propnum,boxnum),:);
    end;
    % add this line to matrix A
    fprintf('%d: ',size(A,1)+1);
    % define bounding sections of entire domain.
    % ASSUMES A SECTION CAN BE SHARED BY TWO AND
    % ONLY TWO BOXES.
    sects=find(sum(geometry,1));
    % loop through sections, loading layer-mean
    % properties
    %std_potmp=[];std_salam=[];
    %for sectnum=1:length(sects);
      %eval(['load ',sectfiles(sects(sectnum),:), ...
          %' LayerPair_potmp LayerPair_salam;']);
      %std_potmp=[std_potmp,LayerPair_potmp];
      %std_salam=[std_salam,LayerPair_salam];
    %end;
    %std_potmp=std(std_potmp')';
    %std_salam=std(std_salam')';
    %mean_salam=fillnans(interp1(glevels, ...
        %meanprop(:,2,boxnum),glayers,'linear'),0);
    %mean_potmp=fillnans(interp1(glevels, ...
        %meanprop(:,3,boxnum),glayers,'linear'),0);
    
    masserror=sqrt(sum(masssecterror(sects).^2));
    errorratio=masserror./sqrt(sum( ...
	modelerror(nlayers+1,1,:).^2));

    if (propnum==1)
      fprintf('Constraining overall net mass.\n')
      error=masserror;
    elseif (propnum==2)
      fprintf('Constraining overall net salt anom.\n');
      error=errorratio*sqrt(sum( ...
	modelerror(nlayers+1,propnum,:).^2));
    else;
      fprintf('Constraining overall net heat anom.\n');
      error=errorratio*sqrt(sum( ...
        modelerror(nlayers+1,propnum,:).^2));
    end;
    A=[A;zeros(1,size(A,2))];
    A(size(A,1),:)=AT;
    b(length(b)+1)=sum(b(indb(nlayers+1,propnum,:)),3);
    Rnn(length(Rnn)+1,length(Rnn)+1)=error^2;
  end;	% loop over properties
end;	% if nboxes>1
clear AT sects std_potmp std_salam mean_potmp mean_salam;

if (0)
% air-sea flux corrections: sum of F* terms must
% be <= 50% of net air-sea flux in each box
for boxnum=1:nboxes;
  for pnum=1:2;
    fprintf('%d: ',size(A,1)+1);
    if (pnum==1) 
      propnum=3; % heat
      er='Constraining net heat flux adjustment';
      er=[er,', box %d.\n'];
      fprintf(er,boxnum);
    else 
      propnum=1; 	% freshwater
      er='Constraining net freshwater flux adjustment';
      er=[er,', box %d.\n'];
      fprintf(er,boxnum);
    end;
    A=[A;zeros(1,size(A,2))];
    A(size(A,1),indF(:,pnum,boxnum))= ...
	ones(1,length(glevels));
    b(length(b)+1)=0;
    Rnn(length(Rnn)+1,length(Rnn)+1)= ...
        (.5*sum(AirSea(:,propnum,boxnum)))^2;
  end;
end;
end;

% top-to-bottom silica conservation
if (consSILCAT)
  for boxnum=1:nboxes;
    fprintf('%d: Conserving top-to-bottom ',size(A,1)+1);
    fprintf('silica, box %d.\n',boxnum);
    Asil=zeros(1,size(A,2));
    bsil=0;
    sects=find(geometry(boxnum,:));
    for sectnum=1:length(sects);
      [Aline,bvalue]=addsilcat(boxnum,sects(sectnum), ...
	sectlabel,lon,lat,D,A,indE,indb,indv,EkmanE);
      Asil=Asil+Aline;
      bsil=bsil+bvalue;
    end;	% loop over bounding sections of box
    A=[A;Asil]; b=[b;bsil-Si_Input(boxnum)];
    Rnn(length(Rnn)+1,length(Rnn)+1)=(SILCATerror)^2;
  end;	% loop over boxes;
end;
clear Aline Asil Bvalue Bsil;

% model-dependent additional constraints
addconstraints;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve the system via Gauss-Markov estimation %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Solving the set of equations.');
fprintf('  Inverting ... ');
time=cputime;
C=Rxx*A'/(A*Rxx*A'+Rnn);
fprintf('done in %5.0f cpu seconds.\n',cputime-time);
% alternative; takes longer in tests on lasalle
%E=A*Rxx*A'+Rnn; [L,U]=lu(E); C=Rxx*A'/U/L;

x=C*b;

% calculate noise, model uncertainty
n=b-A*x;
Pxx=Rxx-C*A*Rxx;
%Pnn=eye(size(Rnn))-A*C;Pnn=Pnn*Rnn*Pnn;
erx=sqrt(diag(Pxx));
clear C;

% define Ax.  A*x=sum(Ax,2).
Ax=A.*(ones(length(b),1)*x');
Aerx=A.*(ones(length(b),1)*erx');

Fdenscorr=NaN*ones(nlayers-1,nboxes);
eFdenscorr=Fdenscorr;
for k=1:nboxes;
  Fdenscorr(:,k)= Fcorr_multfact(:,1,k).*x(indF(:,1,k))+ ...
	Fcorr_multfact(:,2,k).*x(indF(:,2,k));
  eFdenscorr(:,k)=sqrt( ...
	(Fcorr_multfact(:,1,k).*erx(indF(:,1,k)) ).^2 + ...
        (Fcorr_multfact(:,2,k).*erx(indF(:,2,k)) ).^2 );
end;
Fdenscorr=[zeros(1,nboxes);Fdenscorr;zeros(1,nboxes)];
eFdenscorr=[zeros(1,nboxes);eFdenscorr;zeros(1,nboxes)];

% include E* terms in EkmanE
for k=1:nsections;
  EkmanE(:,:,:,k)=EkmanE(:,:,:,k)*(1+x(indE(k)));
end;

% calculate the final diapycnal advection (m/s)
W=W_ap;
for boxnum=1:nboxes;
  W(2:nlayers,boxnum)=W(2:nlayers,boxnum) ...
	-Fdenscorr(2:nlayers,boxnum)./ ...
		level_area(:,boxnum) ...
	+ x(indw(:,1,boxnum));
end;
q=find(isnan(W));W(q)=W_ap(q);

clear Fer Psi Psi2 RV XCW XRW Xbs Xcols Xrows 
clear Xzeros alpha arcPE beta binPair_vel
clear bl box_menu_opt box_menu_stop boxnum
clear col_beg col_end cons ct defum dg
clear diafluxmag directory eFer eqn_beg eqn_end
clear file_Xcols file_Xrows ind j jj k laynum
clear multfact nboxeqns nsurfs pfluxek
clear q q2 rho rho0 row_beg row_end rv
clear rvfilename sarea sbar sectname stdsalt
clear tbar ti tvals wmag wadj

save gaussout

if ( max(abs(x)./sqrt(diag(Rxx)) )>1 | ...
	max(abs(n)./sqrt(diag(Rnn)) )>1)
  disp('*** inversion failed to satisfy a priori choices ***');
end;

%%%%%%%%%%%%%%%%
% Plot results %
%%%%%%%%%%%%%%%%

figure(1);	% compare x and n to priors
compsoln2priors;

break;

figure(2)
propnum=1;
massnum=0;
plotlatflux;

figure(3);	% model solution velocities
clf;
propnum=1;
for boxnum=1:nboxes;
  subplot(2,nboxes,boxnum);
  plotdiaflux;
end;
subplot(212);
plotrefvel;

