
function [Tf,Vdot,V]=flushtime(massnum,boxnum);

% flushtime: a calculation inspired by Worthington's (1976)
% net residence times.  For a layer (massnum=-layernum)
% or block of layers (massnum>=0; defined in waterdiv.m)
% add up the net inputs or outputs (actually take mean,
% as they won't exactly balance in inverse solution)
% for net isopycnal, diapycnal volume transport,
% to get flushing flux Vdot.  Divide layer(s) volume
% V by Vdot, to get net flushing time Tf (years).  Actual
% age of water since ventilation may be considerably less, 
% esp. in WBC (see Worthington's comments).

MN=massnum;
load gaussout;
massnum=MN;
propnum=1;
fluxdivfact=1e6*ones(size(glayers));

if (massnum>0)
  titl=waterdivslab(massnum,:);
elseif (massnum<0)
  titl=['layer ',num2str(-massnum)];
end;
if (massnum==0)
  nlyr=1:length(glayers);
  titl='Net';
elseif (massnum==1)
  nlyr=find(glayers<waterdivs(massnum));
elseif (massnum==length(waterdivs)+1)
  nlyr=find(glayers>=last(waterdivs));
elseif (massnum>1 & massnum<length(waterdivs)+1)
  nlyr=find(glayers>=waterdivs(massnum-1) & ...
        glayers<waterdivs(massnum));
else
  nlyr=-massnum;
end;

% Calculate the lateral flux latflux(boxnum,sectnum,1)
% across each section in layer "massnum".
% This flux is composed of the total geostrophic
% flow (relative + reference) and the Ekman flow.
% latflux(boxnum,sectnum,2) contains the std errors.
latflux=zeros(nsections,2);
for sectnum=1:nsections;
  q=find(sectlabel==sectnum);
  if (length(q))
    pfluxref=sum(Ax(indb(nlyr,propnum,boxnum),indv(q)),2);

    h=A(indb(nlyr,propnum,boxnum),indv(q))';
    h=h./(ones(size(h,1),1)*fluxdivfact(nlyr)');
    eref=sum(sum(h'*Pxx(indv(q),indv(q))*h));
    eref=sqrt(eref);

    pfluxrel=-sum(D(indb(nlyr,propnum,boxnum),indv(q)),2);
    EkIn=EkmanE(nlyr,propnum,boxnum,sectnum);

    pfluxref=pfluxref./fluxdivfact(nlyr);
    pfluxrel=pfluxrel./fluxdivfact(nlyr);
    EkIn=EkIn./fluxdivfact(nlyr);

    latflux(sectnum,1)=sum(pfluxref+pfluxrel+EkIn);
    latflux(sectnum,2)=eref;
  end;
end;

nlvl=[min(nlyr)-1 max(nlyr)];

% calculate airsea(:,1), a vector containing
% the air-sea density flux across the bottom
% and top of the surface.  airsea(:,2) contains
% the std errors.
airsea=NaN*ones(2,2);
airsea(1,1)=( Fdens(nlvl(1)+1,boxnum) + ...
	Fdenscorr(nlvl(1)+1,boxnum));
airsea(1,2)=eFdenscorr(nlvl(1)+1,boxnum);
airsea(2,1)=-(Fdens(nlvl(2)+1,boxnum) + ...
	Fdenscorr(nlvl(2)+1,boxnum));
airsea(2,2)=eFdenscorr(nlvl(2)+1,boxnum);
airsea=airsea/1e6;

% define mixing(1:2,1), a matrix
% containing the diapycnic flux across the lower (1,1)
% and upper (2,1) levels.
mixing=zeros(2,2);
ermixing=mixing;
if (nlvl(1)>0) 
  mixing(1,1)=-x(indw(nlvl(1),propnum,boxnum)).* ...
	meanprop(nlvl(1),propnum,boxnum).* ...
	level_area(nlvl(1),boxnum)/1e6;
  mixing(1,2)=erx(indw(nlvl(1),propnum,boxnum)).* ...
        meanprop(nlvl(1),propnum,boxnum).* ...
	level_area(nlvl(1),boxnum)/1e6;
end;
if (nlvl(2)<nlayers)
  mixing(2,1)=x(indw(nlvl(2),propnum,boxnum)).* ...
        meanprop(nlvl(2),propnum,boxnum).* ...
	level_area(nlvl(2),boxnum)/1e6;
  mixing(2,2)=erx(indw(nlvl(2),propnum,boxnum)).* ...
        meanprop(nlvl(2),propnum,boxnum) .* ...
	level_area(nlvl(2),boxnum)/1e6;
end;

diaflux=NaN*ones(2,2);
for i=1:2;
  diaflux(i,1)=airsea(i,1)+mixing(i,1);
  diaflux(i,2)=sqrt(airsea(i,2).^2 + mixing(i,2).^2);
end;

netflux=[latflux;diaflux];
q=find(netflux(:,1)>0);
Vdot=sum(netflux(q,1));
q=find(netflux(:,1)<0);
Vdot=mean([Vdot -sum(netflux(q,1))]);

% load layer areas, widths
eval(['load mprop_atl',num2str(boxnum),' layer*']);
eval(['A=layer_area_atl',num2str(boxnum),';']);
eval(['H=layer_width_atl',num2str(boxnum),';']);
V=A.*H;
V=nansum(V(nlyr));

Tf=V./Vdot;
eVdot=nanmax(abs(netflux(:,2)));
eTf=[V./(Vdot+nanmax(abs(netflux(:,2))))  ...
	V./(Vdot-nanmax(abs(netflux(:,2))))];
Tf=Tf/1e6 /3600/24/365.24;	% convert to years
eTf=eTf/1e6 /3600/24/365.24; 

V=V/1e6   /(1e3)^3;	% convert to 10^6 km^3

fprintf('\n');
fprintf('Vdot (Sv): %3.1f +- %3.1f\n',Vdot,eVdot);
fprintf('V (1e6 km^3):	%3.2f\n',V);
fprintf('T (yrs): %5.1f (%5.1f - %5.1f)\n',Tf,eTf(1),eTf(2));
fprintf('\n');

