% gauss.m
%
% Use Gauss-Markov estimation to solve the
% system Ax + n = y.
%
% Run after correcting y for air-sea forcing
% and using DOBOX to add diapycnal flux terms
% to A, x.

figure(1);
checkgeo;
pause(.1);

masslayererror=.1;	% layer mass error (Sv)
%min%temp%error=.01;	% minimum temp error (PW)
mindiaflux=1e6;		% minimum dia.vol.flux (m^3/s)

masslayererror=masslayererror*1e6;
%mintemperror=mintemperror*(1e15 /3990/1026);
minsalamerror=35*masslayererror;	% minimum salam error (psu Sv)
minsalamerror=minsalamerror*(1e-3);	% 1e6 already included

% fluxunits: units of terms in b, A*x (after dividing
%	by fluxdivfact).  Untouched units are
%       SI: volume is m^3/s, salam is 1e-3 psu m^3/s,
%	potmp is dgC m^3/s.  Use a mean value of 
%	1e15/(rho*C_p) for potemp to get PW.
fluxunits=['   Sv  '; ...
	   ' psu Sv'; ...
	   '   PW  ']; 
fluxdivfact=[1e6; 1e-3*1e6; 1e15/3985/1026];
propnames=['mass';'salt';'temp'];
propunits=['10^{6} km^2'; ...
	'    psu    '; ...
	'  ^\circC  '];
propdivfact=[1e12;1e-3;1];
propcolr=[[0 0 0];[0 .3 0];[1 0 0]];

% load positions of station pairs
lon2=[];lat2=[];theta2=[];sectlabel=[];R2=[];vmag=[];
for i=1:nsections;
  eval(['load ',sectfiles(i,:),' lon lat']);
  sectt=['refvel_', ...
	sectfiles(i,length(directory)+1: ...
	size(sectfiles,2))];
  eval(['load ',sectt,' Vmag']);
  lon2=[lon2;(lon(1:length(lon)-1)+diff(lon)/2)'];
  lat2=[lat2;(lat(1:length(lat)-1)+diff(lat)/2)'];
  [R,theta]=sw_dist(lat,lon,'km');
  R2=[R2;R'];
  theta2=[theta2;theta'];
  sectlabel=[sectlabel;i*ones(length(lon)-1,1)];
  vmag=[vmag;Vmag'];
end;	

lon=lon2;clear lon2;
lat=lat2;clear lat2;
R=R2*1000;clear R2;	% R [m]
theta=theta2*2*pi/360;clear theta2;	
			% theta [rad]
clear Vmag sectt;

%keyboard;

% define glayers, a vector containing the mean
% gamma of the model layers.
load glevels;
glayers=[glevels(1)-.5*(glevels(2)-glevels(1)); ...
	glevels(1:length(glevels)-1)+diff(glevels)/2; ...
	last(glevels)+last(diff(glevels))/2];
layers=(1:nlayers)';

% define the indices for the variables.
defindices;

disp('Conserved properties are hard-coded. ');
disp('  CHECK: property 1 should be mass,');
disp(['                       and is ',properties(1,:)]);
disp('  CHECK: property 2 should be salam,');
disp(['                       and is ',properties(2,:)]);
disp('  CHECK: property 3 should be potmp or temp,');
disp(['                       and is ',properties(3,:)]);

% Define matrices which contain the surface fluxes and values
%
%  AirSea(layer,property,box) : air-sea formation rate
%  eAirSea(layer,property,box) : error bar range of AirSea
%  EkmanI(layer,property,box) : internal (within each box) 
%				Ekman flux
%  EkmanE(layer,property,box,section)
%			      : external (across-section)
%				Ekman flux
%  SSValue(layer,property,box) : sea surface area, temp, salinity

Fdens=NaN*ones(nlayers+1,nboxes);
AirSea=NaN*ones(nlayers,nproperties,nboxes);
eAirSea=AirSea;EkmanI=AirSea; SSValue=AirSea;
EkmanE=NaN*ones(nlayers,nproperties,nboxes,nsections);
for boxnum=1:nboxes;
  eval(['load airsea',num2str(boxnum),' M* F* Ek* gamma2 SS*']);

  Fdens(:,boxnum)=Fm(1,:)';

  AirSea(:,1,boxnum)=Mm(1,:);
  AirSea(:,2,boxnum)=Ffw(1,:);
  AirSea(:,3,boxnum)=Fh(1,:);

  eAirSea(:,1,boxnum)=Mm(2,:);
  eAirSea(:,2,boxnum)=Ffw(2,:);
  eAirSea(:,3,boxnum)=Fh(2,:);

  EkmanI(:,1,boxnum)=Mmek;
  EkmanI(:,2,boxnum)=Msek;
  EkmanI(:,3,boxnum)=Mhek;

  EkmanE(:,1,boxnum,:)=reshape(EkmanInM, ...
	[nlayers,1,1,nsections]);
  EkmanE(:,2,boxnum,:)=reshape(EkmanInS, ...
	[nlayers,1,1,nsections]);
  EkmanE(:,3,boxnum,:)=reshape(EkmanInH, ...
	[nlayers,1,1,nsections]);

  SSValue(:,1,boxnum)=SSArea;
  SSValue(:,2,boxnum)=SSSbar;
  SSValue(:,3,boxnum)=SSTbar;

end;
clear Mm* Ffw* Fh* Ms* Mh* EkmanInM EkmanInS EkmanInH;
clear SSTbar SSSbar SSArea;

% Find interior sections (defined as sum(geometry,1)==0)
% and insure net Ekman flux out of one box equals flux
% into the other.   This won't be true by default due to
% extrapolation of winds and changes in surface gamma
% across the section.  THIS ROUTINE ASSUMES A SECTION
% CAN BE SHARED BY TWO AND ONLY TWO BOXES.
sectint=find(sum(geometry,1)==0);
for i=1:length(sectint);
  EkM=[];EkS=[];EkH=[];
  for boxnum=1:nboxes;		% calculate mean Ekman flux
    EkM=[EkM,EkmanE(:,1,boxnum,sectint(i)).* ...
	geometry(boxnum,sectint(i))];
    EkS=[EkS,EkmanE(:,2,boxnum,sectint(i)).* ...
        geometry(boxnum,sectint(i))];
    EkH=[EkH,EkmanE(:,3,boxnum,sectint(i)).* ...
        geometry(boxnum,sectint(i))];
  end;
  EkM=mean(EkM,2);EkS=mean(EkS,2);EkH=mean(EkH,2);
  for boxnum=1:nboxes;		% assign mean to each section
    if (geometry(boxnum,sectint(i)))
      EkmanE(:,1,boxnum,sectint(i))= ...
	EkM/geometry(boxnum,sectint(i));
      EkmanE(:,2,boxnum,sectint(i))= ...
        EkS/geometry(boxnum,sectint(i));
      EkmanE(:,3,boxnum,sectint(i))= ...
        EkH/geometry(boxnum,sectint(i));
    end;
  end;
end;
clear sectint EkM EkS EkH;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now include the air-sea formation and %
% Ekman fluxes in vector b		%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

includeEkmanI=0;

disp('Including air-sea fluxes in vector b.');
for j=1:nproperties;
  for k=1:nboxes;

    b(indb(1:nlayers,j,k))= ...
        b(indb(1:nlayers,j,k)) - ( ...
	includeEkmanI*EkmanI(:,j,k)+ ...
        sum(EkmanE(:,j,k,:),4)   );
    b(indb(nlayers+1,j,k))= ...
        b(indb(nlayers+1,j,k)) - nansum( ...
        includeEkmanI*EkmanI(:,j,k)+ ...
        sum(EkmanE(:,j,k,:),4)   );

    % include air-sea density flux and 
    % freshwater flux in mass conservation
    if (j==1)
      b(indb(1:nlayers,j,k))= ...
        b(indb(1:nlayers,j,k)) - AirSea(:,1,k) ...
		-AirSea(:,2,k);
      b(indb(nlayers+1,j,k))= ...
	b(indb(nlayers+1,j,k)) -nansum( ...
	AirSea(:,1,k) +AirSea(:,2,k) );
    end;

    % include heat flux in heat conservation
    if (j==3)
      b(indb(1:nlayers,j,k))= ...
        b(indb(1:nlayers,j,k)) - AirSea(:,3,k);
      b(indb(nlayers+1,j,k))= ...
        b(indb(nlayers+1,j,k)) -nansum(AirSea(:,3,k));
    end;

  end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Include corrections to air-sea forcing %
% in the set of conservation equations   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Including corrections to fluxes in vector x.');
A=[A,zeros(mA,(nlayers-1)*2*nboxes)];

Fcorr_multfact=NaN*ones(nlayers-1,2,nboxes);
dg=diff(glayers);

for k=1:nboxes;

  % add heat flux corrections to heat budget
  % (remember, corrections are defined on glevels
  %  grid; add 1/2 from level i-1, 1/2 from level i.)
  %  for top/bottom layers, use 1/2 value at closest level.)
  A(indb(1,3,k),indF(1,1,k))=.5;
  A(indb(2:nlayers-1,3,k),indF(1:nlayers-2,1,k))= ...
	.5*diag(ones(nlayers-2,1));
  A(indb(2:nlayers-1,3,k),indF(2:nlayers-1,1,k))= ...
        .5*diag(ones(nlayers-2,1));
  A(indb(nlayers,3,k),indF(nlayers-1,1,k))=.5;
  % add sum of layer corrections to total heat budget
  A(indb(nlayers+1,3,k),indF(:,1,k))=ones(1,nlayers-1);

  % add freshwater flux corrections to volume budget
  A(indb(1,1,k),indF(1,2,k))=.5;
  A(indb(2:nlayers-1,1,k),indF(1:nlayers-2,2,k))= ...
        .5*diag(ones(nlayers-2,1));
  A(indb(2:nlayers-1,1,k),indF(2:nlayers-1,2,k))= ...
        .5*diag(ones(nlayers-2,1));
  A(indb(nlayers,1,k),indF(nlayers-1,2,k))=.5;
  % add sum of layer corrections to total volume budget
  A(indb(nlayers+1,1,k),indF(:,2,k))=ones(1,nlayers-1);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % construct a density flux correction from the
  % heat and freshwater flux corrections, and include
  % this in the volume budgets.
  Tbar=interp1(glayers,fillnans(SSValue(:,3,k),0), ...
	glevels,'linear');
  Sbar=interp1(glayers,fillnans(SSValue(:,2,k),0), ...
	glevels,'linear');
  rho=sw_dens(Sbar,Tbar,0);
  rho0=sw_dens(zeros(size(Tbar)),Tbar,0);
  alpha=sw_alpha(Sbar,Tbar,0);
  beta=sw_beta(Sbar,Tbar,0);

  % density flux from heat flux
  Fcorr_multfact(:,1,k)=-alpha.*rho./dg;

  % density flux from freshwater flux:
  % multiply by -1 (fw flux has opposite
  % sign of E-P flux) and don't forget
  % factor of SSS/(1-SSS/1000)
  Fcorr_multfact(:,2,k)=-beta.*rho./dg .*Sbar./(1-Sbar/1000);

  % Loop through layers.  For each layer, add the
  % density flux into the layer at level i-1 and
  % remove the flux out of the layer at level i.
  for i=1:nlayers-1;	% remove flux at glevel(i)
    A(indb(i,1,k),indF(i,1,k))=-Fcorr_multfact(i,1,k);
    A(indb(i,1,k),indF(i,2,k))=-Fcorr_multfact(i,2,k);
  end;
  for i=2:nlayers;	% add flux at glevel(i-1)
    A(indb(i,1,k),indF(i-1,1,k))=Fcorr_multfact(i-1,1,k);
    A(indb(i,1,k),indF(i-1,2,k))=Fcorr_multfact(i-1,2,k);
  end;
  % by construction, these terms sum to zero for
  % vertically-integrated mass balance.

  % done adding volume flux corrections
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end;
nA=size(A,2);

disp(' ');

%%%%%%%%%%%%%%%%%%%%%%%
% Define model errors %
%%%%%%%%%%%%%%%%%%%%%%%

% allocate memory for modelerror
modelerror=NaN*ones(nlayers+1,nproperties,nboxes);

for (boxnum=1:nboxes);

  %%%% mass errors %%%%%
  % set layer errors to masslayererror
  modelerror(1:nlayers,1,boxnum)=masslayererror;
  % assign masstotalerror 
  modelerror(nlayers+1,1,boxnum)=masstotalerror(boxnum);

  % load std devs of salt, temp for this box
  eval(['load mprop_atl',num2str(boxnum), ...
	' stdsalt_atl',num2str(boxnum), ...
	' stdtemp_atl',num2str(boxnum)]);

  %%%% salinity anomaly errors %%%%
  eval(['stdsalt=stdsalt_atl',num2str(boxnum),';']);
  % convert std(salinity) to std(salam) (factor of 1e-3)
  stdsalam=fillnans(interp1(glevels,stdsalt, ...
	glayers,'linear'),0)/1000;
  meansalam=fillnans(interp1(glevels, ...
	meanprop(:,2,boxnum),glayers,'linear'),0);

  % assign (abs(mean) + 2 std)*mass flux error to modelerror
  modelerror(1:nlayers,2,boxnum)= ...
	(abs(meansalam)+2*stdsalam).* ...
	modelerror(1:nlayers,1,boxnum);
  % vert. integrated error: max layer error times
  % ratio of total mass error to layer mass error
  q=find(modelerror(1:nlayers,2,boxnum)== ...
        max(modelerror(1:nlayers,2,boxnum)));
  modelerror(nlayers+1,2,boxnum)= ...
        modelerror(q(1),2,boxnum) ...
        * modelerror(nlayers+1,1,boxnum) ...
        / modelerror(q(1),1,boxnum);
  % set floor to error
  q=find(modelerror(:,2,boxnum)<minsalamerror);
  modelerror(q,2,boxnum)=minsalamerror;
  % if total salam error is < 35*masstotalerror,
  % set it to this value
  if (modelerror(nlayers+1,2,boxnum) < ...
	35*masstotalerror(boxnum)/1e3)
    modelerror(nlayers+1,2,boxnum)= ...
	35*masstotalerror(boxnum)/1e3;
  end;

  %%%% potential temperature errors %%%%
  % error in PW: meanprop(:,3,boxnum)/(1e15/3990/1026)
  eval(['stdtemp=stdtemp_atl',num2str(boxnum),';']);
  stdtemp=fillnans(interp1(glevels,stdtemp,glayers,'linear'),0);
  meantemp=fillnans(interp1(glevels, ...
        meanprop(:,3,boxnum),glayers,'linear'),0);

  % assign (abs(mean) + 2 std)*mass flux error to modelerror
  modelerror(1:nlayers,3,boxnum)= ...
        (abs(meantemp)+2*stdtemp).* ...
        modelerror(1:nlayers,1,boxnum);

  % vert. integrated error: max layer error times
  % ratio of total mass error to layer mass error
  q=find(modelerror(1:nlayers,3,boxnum)== ...
        max(modelerror(1:nlayers,3,boxnum)));
  modelerror(nlayers+1,3,boxnum)= ...
	modelerror(q(1),3,boxnum) ...
	* modelerror(nlayers+1,1,boxnum) ...
	/ modelerror(q(1),1,boxnum);

  eval(['clear *_atl',num2str(boxnum)]);
  clear meansalam stdsalam meantemp stdtemp;

end;

% allocate memory for the noise and model
% variance matrices
Rnn=eye(mA,mA);
Rxx=eye(nA,nA);

% assign squared values to error matrix Rnn
for i=1:nlayers+1;
  for j=1:nproperties;
    for k=1:nboxes;
      Rnn(indb(i,j,k),indb(i,j,k))=modelerror(i,j,k)^2;
    end;
  end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define a priori values for unknowns %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% a priori reference velocities
Rxx(indv,indv)=diag(vmag.^2);

if (0)
% find where station separation distance
% is less than corrdist (km); scale the cross-
% correlation elements of Rxx for these
% pairs by the a priori vmag values,
% scaled by ( 1-dist/corrdist).
corrdist=20;
for stnpair=1:length(lon);
  dist=NaN*ones(size(lon));
  for j=1:length(lon);
    dist(j)=sw_dist([lat(stnpair) lat(j)], ...
	[lon(stnpair) lon(j)],'km');
  end;
  q=find(dist>0 & dist<corrdist);
  for j=1:length(q);
    Rxx(indv(q(j)),indv(stnpair))= ...
	vmag(q(j))*vmag(stnpair) * (1-dist(q(j))/corrdist);
    Rxx(indv(stnpair),indv(q(j)))= ...
	vmag(q(j))*vmag(stnpair) * (1-dist(q(j))/corrdist);
  end;
end;
end;

% a priori diapycnic velocities

for boxnum=1:nboxes;

  propnum=1;
  pfluxrel=-sum(D(indb(1:nlayers,propnum,boxnum),indv),2);
  pfluxek=sum(EkmanE(1:nlayers,propnum,boxnum,:),4);
  Psi=flipud(cumsum(flipud(pfluxrel+pfluxek)));
  Fer=interp1(gamma2,Fdens(:,boxnum),glayers,'linear');
  multfact=0;	%multfact=.25;
	% to do this correctly, MUST cumsum the errors.
	% set multfact=0 until fixed.
  Fer=[Fer+multfact*eAirSea(:,1,boxnum), ...
	Fer-multfact*eAirSea(:,1,boxnum)];
  diafluxmag=abs(-Psi*ones(1,2)-Fer);
  diafluxmag=min(diafluxmag')';
  df=NaN*ones(size(glevels));
  for ti=1:length(df);
    df(ti)=max([diafluxmag(ti) diafluxmag(ti+1)]);
  end;
  diafluxmag=df;
  diafluxmag=fillnans(diafluxmag,0);
  % enforce a minimum of mindiaflux on diafluxmag
  %%%q=find(diafluxmag<mindiaflux & diafluxmag>0);
  %%%diafluxmag(q)=mindiaflux;
  % wmagmass=diafluxmag (m^3/s) / area (m^2) 
  sarea=meanprop(:,1,boxnum);
  wmagmass=diafluxmag./sarea;
  q=find(isnan(wmagmass));
  wmagmass(q)=1e-99;

  propnum=2;
  pfluxrel=-sum(D(indb(1:nlayers,propnum,boxnum),indv),2);
  pfluxek=sum(EkmanE(1:nlayers,propnum,boxnum,:),4);
  Psi=flipud(cumsum(flipud(pfluxrel+pfluxek)));
  % AirSea(:,2,boxnum) is FW flux; doesn't apply to salt budget
  diafluxmag=abs(-Psi);
  df=NaN*ones(size(glevels));
  for ti=1:length(df);
    df(ti)=max([diafluxmag(ti) diafluxmag(ti+1)]);
  end;
  diafluxmag=df;
  diafluxmag=fillnans(diafluxmag,0);
  % enforce a minimum of mindiaflux*<salam> on diafluxmag
  %%%sbar=fillnans(meanprop(:,propnum,boxnum),0);
  %%%q=find(diafluxmag<abs(mindiaflux*sbar) & diafluxmag>0);
  %%%diafluxmag(q)=abs(mindiaflux*sbar(q));
  % wmagsalt=diafluxmag (psu m^3/s) / area (m^2) /Sbar (psu)
  sbar=abs(meanprop(:,2,boxnum));
  wmagsalt=diafluxmag./sarea./sbar;
  q=find(isnan(wmagsalt));
  wmagsalt(q)=1e-99;

  propnum=3;
  pfluxrel=-sum(D(indb(1:nlayers,propnum,boxnum),indv),2);
  pfluxek=sum(EkmanE(1:nlayers,propnum,boxnum,:),4);
  Psi=flipud(cumsum(flipud(pfluxrel+pfluxek)));
  Fer=flipud(cumsum(flipud(AirSea(:,propnum,boxnum))));
  %multfact=.25;
  %Fer=[Fer+multfact*eAirSea(:,propnum,boxnum), ...
        %Fer-multfact*eAirSea(:,propnum,boxnum)];
  %diafluxmag=abs(-Psi*ones(1,2)-Fer);
  %diafluxmag=min(diafluxmag')';
  diafluxmag=abs(-Psi-Fer);
  df=NaN*ones(size(glevels));
  for ti=1:length(df);
    df(ti)=max([diafluxmag(ti) diafluxmag(ti+1)]);
  end;
  diafluxmag=df;
  diafluxmag=fillnans(diafluxmag,0);
  % enforce a minimum of mindiaflux*Tbar on diafluxmag
  %%%tbar=fillnans(meanprop(:,propnum,boxnum),0);
  %%%q=find(diafluxmag<abs(mindiaflux*tbar) & diafluxmag>0);
  %%%diafluxmag(q)=abs(mindiaflux*tbar(q));
  % wmagheat=diafluxmag (degC m^3/s) / area (m^2) /Tbar (degC)
  tbar=abs(meanprop(:,3,boxnum));
  wmagheat=diafluxmag./sarea./tbar;
  q=find(isnan(wmagheat));
  wmagheat(q)=1e-99;

  % Allow for a little "slop" in the system: don't force
  % each and every level to have the minimum mixing according
  % to the a priori choices, so that the system has more
  % freedom to adjust from those a priori values and find
  % a better global minimum for mixing.  This "slop" is
  % mindiaflux (minimum diapycnal volume flux) for mass,
  % mindiaflux*sbar for salt, and mindiaflux*tbar for heat.
  sarea=fillnans(sarea,0);
  wmagmass=(wmagmass.*sarea + mindiaflux)./sarea;
  wmagsalt=(wmagsalt.*sarea + mindiaflux)./sarea;
  wmagheat=(wmagheat.*sarea + mindiaflux)./sarea;

  Rxx(indw(:,1,boxnum),indw(:,1,boxnum))=diag( wmagmass.^2 );
  Rxx(indw(:,2,boxnum),indw(:,2,boxnum))=diag( wmagsalt.^2 );
  Rxx(indw(:,3,boxnum),indw(:,3,boxnum))=diag( wmagheat.^2 );

  % air-sea correction terms
  % heat
  %tvals=interp1(glayers,eAirSea(:,3,boxnum), ...
        %glevels,'linear').^2;
  tvals=NaN*ones(size(glevels));
  for ti=1:length(glevels);
    tvals(ti)=max([eAirSea(ti,3,boxnum) ...
	 eAirSea(ti+1,3,boxnum)].^2);
  end;
  q=find(tvals>0);q2=find(tvals==0);
  tvals(q2)=.01*min(tvals(q));

  Rxx(indF(:,1,boxnum),indF(:,1,boxnum))=diag( tvals );
  % freshwater
  %tvals=interp1(glayers,eAirSea(:,2,boxnum), ...
	%glevels,'linear').^2;
  tvals=NaN*ones(size(glevels));
  for ti=1:length(glevels);
    tvals(ti)=max([eAirSea(ti,2,boxnum) ...
         eAirSea(ti+1,2,boxnum)].^2);
  end;
  q=find(tvals>0);q2=find(tvals==0);
  tvals(q2)=.01*min(tvals(q));
  Rxx(indF(:,2,boxnum),indF(:,2,boxnum))=diag( tvals );
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Add constraints  		%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% model-dependent additional constraints
addconstraints;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve the system via Gauss-Markov estimation %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Solving the set of equations.');
x=(Rxx*A'/(A*Rxx*A'+Rnn))*b;

% calculate noise, model uncertainty
n=b-A*x;
Pxx=Rxx-(Rxx*A'/(A*Rxx*A'+Rnn))*A*Rxx;
erx=sqrt(diag(Pxx));

% define Ax.  A*x=sum(Ax,2).
Ax=A.*(ones(length(b),1)*x');
Aerx=A.*(ones(length(b),1)*erx');

Fdenscorr=NaN*ones(nlayers-1,nboxes);
eFdenscorr=Fdenscorr;
for k=1:nboxes;
  Fdenscorr(:,k)= Fcorr_multfact(:,1,k).*x(indF(:,1,k))+ ...
	Fcorr_multfact(:,2,k).*x(indF(:,2,k));
  eFdenscorr(:,k)=sqrt( ...
	(Fcorr_multfact(:,1,k).*erx(indF(:,1,k)) ).^2 + ...
        (Fcorr_multfact(:,2,k).*erx(indF(:,2,k)) ).^2 );
end;
Fdenscorr=[zeros(1,nboxes);Fdenscorr;zeros(1,nboxes)];
eFdenscorr=[zeros(1,nboxes);eFdenscorr;zeros(1,nboxes)];

save gaussout

%%%%%%%%%%%%%%%%
% Plot results %
%%%%%%%%%%%%%%%%

figure(1);	% compare x and n to priors
subplot(211);
plot(abs(x(indv))./sqrt(diag(Rxx(indv,indv))), ...
	'color',[.5 .5 .5]);
hold on;
for boxnum=1:nboxes;
  for propnum=1:nproperties;
    td=indw(1:nlayers-1,propnum,boxnum);
    plot(td,abs(x(td))./sqrt(diag(Rxx(td,td))), ...
        'color',propcolr(propnum,:));
  end;
  td=indF(1:nlayers-1,1,boxnum);
  plot(td,abs(x(td))./sqrt(diag(Rxx(td,td))), ...
	'.','color',[1 .2 .2], ...
	'markersize',6);
  td=indF(1:nlayers-1,2,boxnum);
  plot(td,abs(x(td))./sqrt(diag(Rxx(td,td))), ...
        '.','color',[.5 .5 1], ...
        'markersize',6);
end;
hold off;
title('x / priors');
subplot(212);
plot(0,0,'.k','markersize',1);
hold on;
for boxnum=1:nboxes;
  for propnum=1:nproperties;
    % layer-by-layer model errors/priors
    td=indb(1:nlayers,propnum,boxnum);
    plot(td,abs(n(td))./sqrt(diag(Rnn(td,td))), ...
	'color',propcolr(propnum,:));
    % vertically-integrated model errors/prior
    td=indb(nlayers+1,propnum,boxnum);
    plot(td-nlayers/2,abs(n(td))./sqrt(diag(Rnn(td,td))), ...
        '.','color',propcolr(propnum,:), ...
	'markersize',20);
    % errors in meeting added constraints/priors
    td=max(max(max(indb)))+1:length(n);
    if (length(td))
      plot(td,abs(n(td))./sqrt(diag(Rnn(td,td))), ...
        'ok','markersize',6);
    end;
  end;
end;
hold off
title('n / priors');

%figure(2);	% plot Ax, n, b for each property
%boxnum=1;
%ploteqnflux;
 
figure(3);	% model solution velocities
clf;
propnum=1;
for boxnum=1:nboxes;
  subplot(2,nboxes,boxnum);
  plotdiaflux;
end;
subplot(212);
plotrefvel;

% flux plots (one figure for each property)
if (0)
for propnum=1:nproperties;

  disp(['Property ',num2str(propnum), ...
	': ',propnames(propnum,:)]);
  figure(propnum+3);
  subplot(131);
  layerplot(meanprop(:,propnum,boxnum)/ ...
	propdivfact(propnum),glevels,'k');
  line([0 0],[0 -max(layers)],'color',[.5 .5 .5]);
  set(gca,'xlim',[min([0 min(meanprop(:,propnum,boxnum))]), ...
	max([0 max(meanprop(:,propnum,boxnum))])]/ ...
	propdivfact(propnum));
  xlabel(['layer mean  (',propunits(propnum,:),')'])
  ylabel('\gamma^n');

% In the model solution, the uncorrected air-sea
% transformations are included in b, while the corrections
% are stored in the F* terms.  Define five new vectors:
%	pfluxlat - lateral property flux (rel+ref+Ek)
%	pfluxref - property flux from reference velocity
%	pfluxrel - property flux from relative velocity
%	pfluxek  - property flux from Ekman drift
%	pfluxdia - diapcynic property flux 
%	pfluxas  - air-sea driven flux (including corrections)
%	pfluxn	 - residual flux (contained in n)
% For all fluxes, positive values represent fluxes
% into the box.
% By definition, pfluxlat(:,1)+pfluxdia(:,1)+pfluxas(:,1)+pfluxn=0.

  pfluxref=sum(Ax(indb(1:nlayers,propnum,boxnum),indv),2) ...
	/fluxdivfact(propnum);
  pfluxref(:,2)=sqrt(sum( ...
	Aerx(indb(1:nlayers,propnum,boxnum),indv).^2,2)) ...
	/fluxdivfact(propnum);
  pfluxrel=-sum(D(indb(1:nlayers,propnum,boxnum),indv),2) ...
	/fluxdivfact(propnum);
  pfluxek=sum(EkmanE(1:nlayers,propnum,boxnum,:),4) ...
	/fluxdivfact(propnum);
  pfluxlat=pfluxref(:,1)+pfluxrel+pfluxek;
  pfluxas=AirSea(1:nlayers,propnum,boxnum)/fluxdivfact(propnum);
  pfluxas(:,2)=eAirSea(1:nlayers,propnum,boxnum) ...
	/fluxdivfact(propnum);
  pfluxascorr=pfluxas(:,1)+n(indb(1:nlayers,propnum,boxnum)) ...
	/fluxdivfact(propnum);
  pfluxdia=sum(Ax(indb(1:nlayers,propnum,boxnum), ...
	indw(:,propnum,boxnum)),2)/fluxdivfact(propnum);
  pfluxdia(:,2)=sum(Aerx(indb(1:nlayers,propnum,boxnum), ...
        indw(:,propnum,boxnum)),2)/fluxdivfact(propnum);

  subplot(132)
  layerplot(pfluxdia(:,1),glayers,'k');
  hold on;
  patch([pfluxlat;zeros(size(pfluxlat))],[-(1:length(glayers))'; ...
        (-length(glayers):-1)'],[.5 .5 1],'edgecolor','none');
  patch([pfluxascorr;zeros(size(pfluxascorr))],[-(1:length(glayers))'; ...
        (-length(glayers):-1)'],[1 .5 .5],'edgecolor','none');
  patch([pfluxdia(:,1);zeros(size(pfluxdia(:,1)))], ...
	[-(1:length(glayers))'; ...
        (-length(glayers):-1)'],[.25 .25 .25],'edgecolor','none');
  plot(pfluxascorr,-layers,'r');
  plot(pfluxlat,-layers,'b');
  meanlayerplot(pfluxlat,pfluxref(:,2),glayers,'b');
  %meanlayerplot(pfluxdia(:,1),pfluxdia(:,2),glayers,'k');
  hold off;
  lim=max([max(abs(sum(pfluxdia,2))) max(abs(pfluxlat+pfluxref(:,2))) ...
	 max(abs(pfluxascorr))]);
  set(gca,'xlim',[-lim  lim]);
  title(propnames(propnum,:));
  xlabel(fluxunits(propnum,:));

  subplot(133);
  wfluxi=[flipud(cumsum(flipud(pfluxdia(:,1))));0];
  pfluxi=[flipud(cumsum(flipud(pfluxlat(:,1))));0];
  Mcorri=[flipud(cumsum(flipud(pfluxascorr)));0];
  fmtop=[flipud(cumsum(flipud(pfluxascorr+pfluxas(:,2))));0];
  fmbot=[flipud(cumsum(flipud(pfluxascorr-pfluxas(:,2))));0];
%%
% don't cumsum std errors!  Must take sqrt(sum(errors^2)))
fmtop=NaN*ones(size(fmtop));fmbot=NaN*ones(size(fmbot));
%%
  layerplot(wfluxi,gamma2,'k');
  hold on;
  patch([fmtop;flipud(fmbot)],[-(1:length(gamma2))'; ...
        (-length(gamma2):-1)'],[1 .5 .4], ...
        'edgecolor','none');
  layerplot(wfluxi,gamma2,'k');
  layerplot(pfluxi,gamma2,'b');
  plot(Mcorri,-(1:length(gamma2)),'r');
  hold off;
  lim=max([max(abs(wfluxi)) max(abs(pfluxi)) ...
	 max(abs(fmtop)) max(abs(fmbot))]);
  set(gca,'xlim',[-lim  lim]);
  xlabel(fluxunits(propnum,:));

end;	% loop over properties
end;

figure(4)
propnum=1;massnum=0;plotlatflux;
if (0)
  subplot(221);massnum=2;plotlatflux;
  subplot(222);massnum=4;plotlatflux;
  subplot(223);massnum=5;plotlatflux;
  subplot(224);massnum=6;plotlatflux;
end;

%heatflux=x(indF(:,1,6))+interp1( ...
  %glayers,AirSea(:,1,6),glevels);
